==> pub_mirror-1.0.6.tar.gz: lib/http.dart <==
import 'dart:io' as io;
import 'dart:math' as math;

import './logging.dart';

class StatusCodeException implements Exception {
  final String url;
  final int statusCode;
  final String reason;

  StatusCodeException({
    this.url = 'some page',
    this.statusCode,
    this.reason,
  });

  @override
  String toString() {
    var buffer = StringBuffer('failed');
    if (statusCode != null) {
      buffer.write(' with ${statusCode}');
    }
    buffer.write(' when visiting ${url}');
    if (reason != null) {
      buffer.write(': ${reason}');
    }
    return buffer.toString();
  }
}

String getSize(num byte) {
  final units = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
  for (int i = 0; i < units.length; i++) {
    final base = math.pow(1024, i);
    if (byte < base * 1024 || i + 1 == units.length) {
      return '${(byte / base).toStringAsFixed(2)} ${units[i]}';
    }
  }
  return 'NaN';
}

/// download file progressively
Future saveFileTo(String url, String destination, {io.HttpClient client}) async {
  if (client == null) {
    client = io.HttpClient();
  }
  var request = await client.getUrl(Uri.parse(url)).timeout(Duration(seconds: 10));
  logger.fine("Connection has been established: ${url}");
  var response = await request.close().timeout(Duration(seconds: 10));
  logger.fine("Response has been received: ${url}");
  if (response.statusCode >= 400) {
    logger.fine("Non-2xx status code: ${url}");
    await response.drain();
    throw StatusCodeException(
      reason: response.reasonPhrase,
      url: url,
      statusCode: response.statusCode,
    );
  }

  final io_sink = io.File(destination).openWrite();
  final watch = Stopwatch()..start();
  var downloaded_length = 0;
  final progress_printer = Stream.periodic(Duration(seconds: 1)).listen((_) {
    final seconds = watch.elapsed.inSeconds;
    logger.info('[${url} ===> ${destination}] ${(100 * downloaded_length / response.contentLength).toStringAsFixed(2)}% Total: ${getSize(response.contentLength)} Downloaded: ${getSize(downloaded_length)} Speed: ${getSize(downloaded_length/seconds)}/s');
  });
  try {
    await for (var block in response.timeout(Duration(seconds: 10))) {
      io_sink.add(block);
      downloaded_length += block.length;
    }
  } catch (e) {
    print('Unhandled exception during downloading: $e');
    rethrow;
  } finally {
    await io_sink.close();
    await progress_printer.cancel();
  }
  assert(downloaded_length == response.contentLength);
  logger.info('[${url} ===> ${destination}] ${getSize(downloaded_length)} saved.');
}

==> pub_mirror-1.0.7.tar.gz: lib/http.dart <==
import 'dart:io' as io;
import 'dart:math' as math;
import 'dart:async' as async;
import 'package:quiver/async.dart' as quiver_async;
import 'package:retry/retry.dart' as retry;

import './logging.dart';

class StatusCodeException implements Exception {
  final String url;
  final int statusCode;
  final String reason;

  StatusCodeException({
    this.url = 'some page',
    this.statusCode,
    this.reason,
  });

  @override
  String toString() {
    var buffer = StringBuffer('failed');
    if (statusCode != null) {
      buffer.write(' with ${statusCode}');
    }
    buffer.write(' when visiting ${url}');
    if (reason != null) {
      buffer.write(': ${reason}');
    }
    return buffer.toString();
  }
}

String getSize(num byte) {
  final units = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
  for (int i = 0; i < units.length; i++) {
    final base = math.pow(1024, i);
    if (byte < base * 1024 || i + 1 == units.length) {
      return '${(byte / base).toStringAsFixed(2)} ${units[i]}';
    }
  }
  return 'NaN';
}

/// download file progressively
Future saveFileTo(String url, String destination, {io.HttpClient client}) async {
  if (client == null) {
    client = io.HttpClient();
  }

  io.HttpClientResponse response = await retry.RetryOptions(maxAttempts: 3).retry(() async {
    logger.fine("Connecting to ${url}");
    final request = await client.getUrl(Uri.parse(url)).timeout(Duration(seconds: 10));
    logger.fine("Connection has been established: ${url}");
    return await request.close().timeout(Duration(seconds: 10));
  }, retryIf: (e) => e is async.TimeoutException);
  logger.fine("Response has been received: ${url}");

  if (response.statusCode >= 400) {
    logger.fine("Non-2xx status code: ${url}");
    await response.drain();
    throw StatusCodeException(
      reason: response.reasonPhrase,
      url: url,
      statusCode: response.statusCode,
    );
  }

  // save response to buffer
  final watch = Stopwatch()..start();
  final buffer = quiver_async.StreamBuffer<int>();
  final progress_printer = Stream.periodic(Duration(seconds: 1)).listen((_) {
    final seconds = watch.elapsed.inSeconds;
    final downloaded_length = buffer.buffered;
    logger.info('[${url} ===> ${destination}] ${(100 * downloaded_length / response.contentLength).toStringAsFixed(2)}% Total: ${getSize(response.contentLength)} Downloaded: ${getSize(downloaded_length)} Speed: ${getSize(downloaded_length/seconds)}/s');
  });
  try {
    await buffer.addStream(response.timeout(Duration(seconds: 10)));
    await buffer.close();
  } catch (e) {
    print('Unhandled exception during downloading: $e');
    rethrow;
  } finally {
    await progress_printer.cancel();
  }
  assert(buffer.buffered == response.contentLength);

  // save content in buffer to file
  final io_sink = io.File(destination).openWrite();
  io_sink.add(await buffer.read(buffer.buffered));
  await io_sink.close();

  logger.info('[${url} ===> ${destination}] ${getSize(response.contentLength)}(${getSize(response.contentLength/watch.elapsed.inSeconds)}/s) saved.');
}
